
Application Integration Server Example Implementation

# Overview

The Application Integration Server Example Implementation is written in Python.
It has an extensible plugin interface, supports TLS encryption,
HTTP Basic Authentication and many more features.


# Files

```text
root
|-- readme.md                   # this file
|-- package.sh                  # create a zip file under linux
|-- package.ps1                 # create a zip file under windows (do not use, has limitations)
|-- Srvany.zip                  # can be used to crate a windows server, see "Running as a Service"
|-- images
    |-- AppIntService[1-4].png  # images used by this file
|-src
    |-- main.py                 # the main executable
    |-- config.ini              # contains the configuration and its explanation
    |-- AppInt.pem              # the X509 certificate and the private key to secure the TLS connection
    |-Plugins                   # the plugin directory
        |-- exCSVSimple.py      # a simple CSV plugin
        |-- exINI.py            # a simple INI file plugin
        |-- exSimple.py         # a simple plugin without external dependencies
        |-- ...                 # and many more
```

# TLS & Certificates

The server supports unencrypted (HTTP) and TLS encrypted (HTTPS) connections, but not both at the same time.
It uses TLS 1.2, which cannot be configured.
The X509 certificate and the private key must be stored in one file in PEM (BASE64) format.
The private key must not be password protected.
Any intermediate certificates must be stored in the same file.
The included certificate MUST be replaced in production with a new one based on a different private key.

# Using in MailStore

By default the server is listening on port 2020 on all network interfaces.
TLS encryption is enabled by default and it is highly recommended to keep it enabled.
The names of the plugins must be used as HTTP path, but without the file extension.

Examples:

To reach the exCSVSimple.py plugin the following URLS has to be used:

`https://127.0.0.1/exCSVSimple`

# Testing in Browser

The Application Integration protocol only specifies the HTTP `POST` verb.
This implementation however also supports HTTP `GET` requests.
This makes it possible to test plugins with any internet browser,
just enter the plugin URL in the address bar of the browser, e.g.:

`https://127.0.0.1/exCSVSimple?cmd=list`

or

`https://127.0.0.1/exCSVSimple?cmd=auth&user=username&pass=password`

You may have to ignore the HTTPS warning and have to enter valid plugin credentials.

# Plugin Features

## Mandatory methods

Any plugin must implement at least a `list_` and an `auth` method.
The `list_` method must return a list of user dictionary objects.
The HTTP query parameters are passed to `list_`, so that additional filtering can be performed on plugin side.
E.g. the exCSVGroups plugin accepts a `group` parameter which allows filtering for user groups.
The exSimple plugin defines some of these user objects and additionally add a 'password' property which is used
for authenticating a user.
The `auth` method accepts a username and a password and returns a dictionary of the form `{'succeeded': True/False}`
The plugin may perform some initialization in its `__init__` method.

## HTTP Basic authentication

When a plugin implements a `http_authentication` method the administrator must enter the required credentials
in the MailStore directory service configuration.
The HTTP authentication itself is completely independent from the user authentication.
One could define the required credentials in the plugin itself, as exINIHash or exSimple do.
This method must return `True` or `False` to indicate successful authentication.

## Global Logger

The main program defines a global logger named `AppIntLogger`.
Plugins can use this logger to write to the same log targets and have the same log levels as the main program.

Example:
```python
import logging

class Plugin:
    def __init__(self):
        self.logger = logging.getLogger("AppIntLogger")
        self.logger.info("init reached")
```
The plugin `exCSVGroups` uses this feature.

## Hashed Passwords

Passwords can be stored hashed. The `exINIHash` plugin uses this feature.

## Groups and other filters

HTTP query parameters are passed to the `list_` method. The plugin is then able to perform further filtering.
The exCSVGroups plugin uses this feature. To filter for a group the following URL must be used in Mailstore:

`https://127.0.0.1:2020/exCSVGroups?group=female`

## Disabled Plugins

To disable a plugin prepend an underscore `_` to the plugin file name, e.g. `_exMariaDB.py`.

## Running as a Service

* Install Python 3.2 or newer.
* Download the [Application Integration Example Implementation](https://help.mailstore.com/en/server/Implementing_an_Application_Integration_Server#Example_Server_Implementations) and extract it into a folder, e.g. c:\MailStoreAppInt.
* Download [Srvany.zip](https://srv001.msintra.net/wiki/Datei:Srvany.zip) and extract it into the same folder, e.g. c:\MailStoreAppInt.
* Open an elevated Powershell and execute the following command, adjust the path to srvany
```
New-Service -Name "MailStoreAppInt" -BinaryPathName "c:\MailStoreAppInt\srvany.exe" -DisplayName "MailStore Application Integration Service" -StartupType Automatic
```
* Open `regedit` and navigate to `HKLM\SYSTEM\CurrentControlSet\Services\MailStoreAppInt`
* Create a new subkey `Parameters`
* Create a new String/REG_SZ `Application` with the value `c:\program files\python37\pythonw.exe`. Use your Python installation path.
* Create a new String/REG_SZ `AppDirectory` with the value `c:\MailStoreAppInt`
* Create a new String/REG_SZ `AppParameters` with the value `c:\MailStoreAppInt\main.py`
* Now the service can be started by using `Services`/`Dienste`

![AppIntService1](images/AppIntService1.png)
![AppIntService2](images/AppIntService2.png)
![AppIntService3](images/AppIntService3.png)
![AppIntService4](images/AppIntService4.png)
