# -*- coding: utf-8 -*-
#
# ApplicationIntegrationServer - AppInt Example Plugin
# This plugin connects to another Application Integration server/plugin/url.
# It could be use to bypass authentication or as reverse proxy.
# It is also a minimal client implementation, which can be used to test other Application Integration servers.
#
# Copyright (c) 2014 - 2017 MailStore Software GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

import urllib.request
import urllib.parse
import base64
import json
import ssl

class Plugin:
    def __init__(self):
        # credentials to use this plugin
        self.plugin_user = "admin"
        self.plugin_password = "password"
        # set remote_user and remote_pass to "" (empty string) to disable authentication
        self.remote_user = "admin"
        self.remote_pass = "password"
        # http and https are supported
        self.remote_url = "https://localhost:2020/exSimple"

    def _create_request(self, values):
        post_data = urllib.parse.urlencode(values).encode()
        request = urllib.request.Request(self.remote_url, post_data)
        if self.remote_user and self.remote_pass:
            auth_header = base64.b64encode("{}:{}".format(self.remote_user, self.remote_pass).encode()).decode()
            request.add_header("Authorization", "Basic {}".format(auth_header))
        answer = urllib.request.urlopen(request, context=ssl._create_unverified_context()).read()
        answer = answer.decode()
        return json.loads(answer)

    def list_(self, _):
        values = [("cmd", "list")]
        return self._create_request(values)

    def auth(self, username, password):
        values = [("cmd", "auth"), ("user", username), ("pass", password)]
        return self._create_request(values)

    def http_authentication(self, username, password):
        return username == self.plugin_user and password == self.plugin_password
