# -*- coding: utf-8 -*-
#
# ApplicationIntegrationServer - INI Example Plugin
#
# Copyright (c) 2017 MailStore Software GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

# https://docs.python.org/3/library/configparser.html
import configparser
import os.path


class Plugin:
    def __init__(self):
        self.ini_file_name = "exINI.ini"
        self.ini_file_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), self.ini_file_name)
        self.conf = configparser.ConfigParser()
        # credentials to use this plugin
        self.plugin_user = "admin"
        self.plugin_password = "password"

    def list_(self, _):
        self.conf.read(self.ini_file_path)
        return [{"userName": section,
                 "fullName": self.conf[section]["fullName"],
                 "emailAddresses": self.conf[section]["emailAddresses"].split(",")}
                for section in self.conf.sections()]

    def auth(self, username, password):
        self.conf.read(self.ini_file_path)
        for section in self.conf.sections():
            if section == username and self.conf[section]["password"] == password:
                return {"succeeded": True}
        return {"succeeded": False}

    def http_authentication(self, username, password):
        return username == self.plugin_user and password == self.plugin_password
