# -*- coding: utf-8 -*-
#
# ApplicationIntegrationServer - MailStore API/HTTP Example Plugin
# This plugin can be used to synchronize and authenticate against another MailStore Server
#
# Copyright (c) 2014 - 2017 MailStore Software GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

# http://en.help.mailstore.com/Python_API_Wrapper_Tutorial
# http://en.help.mailstore.com/Administration_API_-_Function_Reference
import mailstore
import base64
import urllib.parse
import urllib.request
import ssl

class Plugin:
    def __init__(self):
        self.mailstore_host = "localhost"
        self.mailstore_mgmt_port = 8463
        self.mailstore_web_port = 8462
        self.mailstore_web_tls = True
        self.mailstore_user = "admin"
        self.mailstore_password = "Passw0rd"
        self.mailstore_ignore_invalid_certificates = True
        self.excluded_users = ["admin"]
        # credentials to use this plugin
        self.plugin_user = "admin"
        self.plugin_password = "password"

    def list_(self, _):
        mailstore_api = mailstore.mgmt.ServerClient(self.mailstore_user, self.mailstore_password,
                                                self.mailstore_host, self.mailstore_mgmt_port,
                                                ignoreInvalidSSLCerts=self.mailstore_ignore_invalid_certificates)
        mailstore_users = mailstore_api.GetUsers()["result"]
        users = list()
        for user in mailstore_users:
            if user["userName"] not in self.excluded_users:
                user["emailAddresses"] = mailstore_api.GetUserInfo(user["userName"])["result"]["emailAddresses"]
                users.append(user)
        return users

    def auth(self, username, password):
        web_url = "{}://{}:{}/b".format("https" if self.mailstore_web_tls else "http",
                                        self.mailstore_host,
                                        self.mailstore_web_port)
        request = urllib.request.Request(web_url)
        auth_header = base64.b64encode("{}:{}".format(username, password).encode()).decode()
        request.add_header("Authorization", "Basic {}".format(auth_header))
        response = urllib.request.urlopen(request, context=ssl._create_unverified_context())
        return {"succeeded": response.code == 200}

    def http_authentication(self, username, password):
        return username == self.plugin_user and password == self.plugin_password
