# -*- coding: utf-8 -*-
#
# ApplicationIntegrationServer - MariaDB/MySQL Example Plugin
#
# Copyright (c) 2017 MailStore Software GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

# MariaDB and MySQL implement (nearly) the same client interface
import MySQLdb


class Plugin:
    def __init__(self):
        self.db_host = "localhost"
        self.db_port = 3306
        self.db_user = "admin"
        self.db_pass = "password"
        self.db_database = "users"
        self.db_table = "users"
        self.fieldnames_list = ["userName", "fullName", "emailAddresses"]
        self.fieldnames_auth = ["userName", "password"]

    def http_authentication(self, username, password):
        return username == self.db_user and password == self.db_pass

    def list_(self, _):
        connection = MySQLdb.connect(host=self.db_host, port=self.db_port, user=self.db_user,
                                     password=self.db_pass, database=self.db_database)
        cursor = connection.cursor()
        cursor.execute('SELECT {} FROM {}'.format(",".join(self.fieldnames_list), self.db_table))
        userlist = [{"userName": user[0],
                     "fullName": user[1],
                     "emailAddresses": user[2].split(",")} for user in cursor.fetchall()]
        connection.close()
        return userlist

    def auth(self, username, password):
        connection = MySQLdb.connect(host=self.db_host, port=self.db_port, user=self.db_user,
                                     password=self.db_pass, database=self.db_database)
        cursor = connection.cursor()
        cursor.execute('SELECT {} FROM {} WHERE userName=%s AND password=%s'.format(",".join(self.fieldnames_auth),
                                                                                    self.db_table),
                       (username, password))
        result = cursor.rowcount == 1
        connection.close()
        return {"succeeded": result}
