# bulkImport.py - Imports all email files of all users.
#
# Copyright (c) 2012 - 2017 MailStore Software GmbH
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

# SERVER CMD on x64 OS
MAILSTORECMD = r"C:\Program Files (x86)\deepinvent\MailStore Server\MailStoreCmdSilent.exe"

# SERVER CMD on x86 OS
# MAILSTORECMD = r"C:\Program Files\deepinvent\MailStore Server\MailStoreCmdSilent.exe"

# CLIENT CMD on x64 OS
# MAILSTORECMD = r"C:\Program Files (x86)\deepinvent\MailStore Client MSI\MailStoreCmdSilent.exe"

# CLIENT CMD on x86 OS
# MAILSTORECMD = r"C:\Program Files\deepinvent\MailStore Client MSI\MailStoreCmdSilent.exe"

# SPE CMD
# MAILSTORECMD = r"C:\Program Files (x86)\E-mail Archive Client\MailStoreCmdSilent.exe"

# SERVER HOST
HOST = "172.20.13.7"
# SPE HOST
# HOST = "https://cas.mailstore-spe.tld:443/instanceid"

# USER and PASS of an administrator
USER = "admin"
PASS = "admin"

# location of the folders containing the user's pst/emlmsg/mbox files
# has to be the same as in createUserFolders.py
# please note the 'r' in the beginning, this indicates a raw-string, \ will not be interpreted
# so, just leave the 'r' in front
rootpath = r"c:\rootpath"

# which bulk import do you want to start? pst, emlmsg or mbox?
filetype = "pst"

# ################## DON'T CHANGE ANYTHING BELOW THIS LINE ###################
import subprocess
import os
import shlex


if not os.path.exists(MAILSTORECMD):
    print("\"{}\" does not exist. Exiting.\nPlease Check the MAILSTORECMD variable.".format(MAILSTORECMD))
    exit(1)


def execute_command(cmd):
    command = "\"" + MAILSTORECMD + "\"" + " --h=\"" + HOST + "\" --u=\"" + USER + "\" --p=\"" + PASS +\
              "\" --pkv3=*" + " --nologo" + " --verbose" + cmd
    try:
        output = subprocess.check_call(shlex.split(command))
    except subprocess.CalledProcessError as e:
        print(" exitcode: {0}".format(e.returncode))
        print("output    : {0}".format(e.output))
        print("command   : {0}".format(command))
    else:
        print(" exitcode: {0}".format(output))


def bulk_import():
    for username in os.listdir(rootpath):
        userpath = os.path.join(rootpath, username)
        # Convert username to lowercase, since MailStore only supports these
        username = username.lower()
        if os.path.isdir(userpath):
            print("USER {0}".format(username))
            if filetype == "emlmsg":
                logname = os.path.join(rootpath, "{DATE}-{TIME}-" + username + ".log")
                # we have to add additional "\" or else the path cannot be found by cmd
                if userpath.startswith("\\"):
                    userpath = "\\\\" + userpath
                    logname = "\\\\" + logname
                execute_command(" --o=\"" + logname + "\""
                                " -c import-execute" +
                                " --name=\"templateBulkImportEMLMSG\"" +
                                " --[directory]=\"" + userpath + "\""
                                " --[recurse]=1"
                                " --[readXMailStoreHeaders]=0" +
                                " --[targetUserFolder]=\"" + username + "\"")
            elif filetype == "pst":
                for foldername, subfolders, files in os.walk(userpath):
                    for file in files:
                        filepath = os.path.join(foldername, file)
                        if filepath.lower().endswith(".pst"):
                            print("FILE {0}".format(filepath), end='\n')
                            logname = os.path.join(rootpath,
                                                   "{DATE}-{TIME}-" +
                                                   filepath.replace(rootpath, "").replace("\\", "-") + ".log")
                            # we have to add additional "\" or else the path cannot be found by cmd
                            if filepath.startswith("\\"):
                                filepath = "\\\\" + filepath
                                logname = "\\\\" + logname
                            execute_command(" --o=\"" + logname + "\"" + " -c import-execute" +
                                            " --name=\"templateBulkImportPST\"" +
                                            " --[targetUserFolder]=\"" + username + "\"" +
                                            " --[outlookProfile]=\"" + filepath + "\"")

            elif filetype == "mbox":
                for file in os.listdir(userpath):
                    filepath = os.path.join(userpath, file)
                    print("FILE {0}".format(filepath), end='')
                    logname = os.path.join(rootpath,
                                           "{DATE}-{TIME}-" + username + "-" + os.path.basename(filepath) + ".log")
                    # we have to add additional "\" or else the path cannot be found by cmd
                    if filepath.startswith("\\"):
                        filepath = "\\\\" + filepath
                        logname = "\\\\" + logname
                    execute_command(" --o=\"" + logname + "\""
                                    " -c import-execute" +
                                    " --name=\"templateBulkImportMBOX\"" +
                                    " --[targetUserFolder]=\"" + username + "\"" +
                                    " --[path]=\"" + filepath + "\"")
            else:
                print("ERROR: filetype not valid. Should be pst, emlmsg or mbox, but is \"{0}\"".format(filetype))

bulk_import()
print("done")
