# Domain Update Scripts #

#### These scripts can be used in the following scenarios:**

* When users in the directory service change
  * wedding or other reason for a name change
  * company policy
* When email addresses of users change
* When directory service settings change
  * Switch from one directory service to another one
  * Migrate from Exchange to Microsoft 365
* When the directory service itself changes
  * Change username format in MailStore from sAMAccountname to UPN or UPN (local part) or vice versa

#### The most common scenarios handled by these scripts:

* The customer has not changed anything and has contacted support beforehand
* The customer already has synchronized moved or renamed users, and the old users are not in MailStore anymore
* The customer already has synchronized moved or renamed users, and the old users are still in MailStore
  * Probably uncommon because twice the number of licenses is required
  * Documentation for this scenario is not included


#### Depending on the scenario, one or more of the following actions must be performed:

* The user itself must be renamed
* The archive with the same name must be renamed
* If there is an archive without a user of the same name, that archive must be renamed
* The user privileges of this user on his own archive must be adjusted
  * The permission must be read (read, write, delete)
  * A new permission must be set accordingly
  * The old permission must be removed
* The permissions of all other users on that users' archive must be adjusted, even when that other user was not renamed at all
  * same steps as above
* Sub folders on the first level must be renamed
  * In case of Exchange the folders are named "Exchange <username>", and the new name must be "Exchange <new_username>"
  
In some cases not all steps can be performed automatically, because old data is missing and cannot be assumed and applied.

## Preparation of MailStore ##

### Final User Synchronization ###

The admin should perform a final user synchronization with the old settings before any change on external systems are made to ensure that all user data is current.

### Backup ###

Before the scripts are used, a valid backup must have been created of the archive.\
It is also a good advice to test these scripts on a test archive on a separate server first.

### Maintenance Mode ###

The execution of these scripts should be done in a maintenance window.
MailStore Server or the SPE instance should be started in *Safe Mode* when these scripts are executed.

Generally, the scripts should work in the normal mode as well, but the behavior of the script is undefined when a time out error occurs, and the scripts are interrupted unexpectedly.
  
## How to use the scripts ## 

There are six scripts for MailStore Server (MSS_`*`) and six scripts for the SPE (SPE_`*`).\
Depending on the scenario of the customer only a few of these scripts are required.

The admin has to open the scripts with a text editor or the PowerShell ISE and has to change the default username, the server name and some other values in some cases.
When the password variable remains empty, an interactive prompt asks for it. This is the preferred usage to avoid password leakage.

The admin may change something below the line that contains FUNCTIONS CAN BE ADJUSTED TO MODIFY FOLDER I/O
Only a few scripts do have such functions.

The admin must not change anything below the line that contains DO NOT CHANGE ANYTHING BELOW THIS LINE.

The scripts can either be executed by double-clicking on them or loading them into PowerShell ISE and starting them from there via F5.

### Prepare ###

Scripts always come in pairs. The first scripts read data from MailStore, such as usernames, archive names and folder names, and prepares a text file with the data gathered. These scripts do have the word "prepare" in them. The file is stored in the root folder of the users' home directory.\
After that the admin has to edit this text file. Since only the admin knows the new username after the migration, this is a manual step. The format after the edit is *old_username=new_username*. One username pair per line.

Example of such a text file before being edited by the admin:

	abby.hernandez=abby.hernandez
	bill.burke=bill.burke
	jeremy.summer=jeremy.summer

When it comes to folders, it looks like this:

	Exchange abby.hernandez=Exchange abby.hernandez
	Exchange bill.burke=Exchange bill.burke
	Exchange jeremy.summer=Exchange jeremy.summer

After editing the file could look like this:

	abby.hernandez=ahernandez
	bill.burke=bburke
	jeremy.summer=jsummer

or

	Exchange abby.hernandez=Exchange ahernandez
	Exchange bill.burke=Exchange bburke
	Exchange jeremy.summer=Exchange jsummer

Additionally, these rules apply:

	#jeremy.summer=lines_starting_with_#_will_be_ignored_in_the_update_step
	completely_empty_lines_will_ignored
	empty_new_names_will_be_ignored=
	=empty_old_names_will_be_ignored
	lines_without_equals_char_will_be_ignored
	equal_entries_will_be_ignored=equal_entries_will_be_ignored

The intention here is to save some write work for the admin, because only one part of the folder name usually changes. Same ignore rules as for users apply here.
Again, the admin has to do some manual work.

### Update ###

When the text files have been edited by the admin, the second script of each pair must be adjusted in regard to credentials and server data and started.\
It reads the file created in the first step and applies the changes to the archive. These scripts do have the word "update" in them.

## Scenarios ##

### 1 Planned Update ###

#### Prepare username changes ####

Adjust the script MSS_1_prepare_users.ps1 matching your installation.
Execute the adjusted script.
The users file is opened automatically in the default editor and can be adjusted according to your planned changes.

#### Apply username changes ####

Adjust the script MSS_2_update_users.ps1 matching your installation.
Execute the adjusted script.

This script will do:

* Read the user mapping from the text file
* Users from MailStore are read and stored
* For each MailStore User from the installation do the following:
  * When the desired new username already exists in MailStore, all operations are skipped
  * Read folder privileges for this user
    * When an archive should be renamed according to text file, adjust privileges
  * The user will be renamed
  * The archive with the same name will be renamed
* Archives from MailStore are read and stored
* For each mapping in the file do the following:
  * If the new archive name already exists, skip renaming
  * Rename archive according to mapping

The order of these commands is critical and must only be changed after heavy reviewing of the process.
The script does not merge users or archives. When a conflict occurs, the script skips the rest of the actions for this mapping.

#### Synchronize with the directory service ####

Depending on your scenario, either change the directory service and synchronize users or only synchronize users. The Sync overview should not show new users. Users may appear as modified when email addresses, full names or other synchronized values change. This is considered uncritical.

Verify your user and archive settings and permissions.

#### Restart Server ####

Since MailStore Server should be running in Safe Mode, users are not able to log in. Restart the MailStore Server service in the normal mode and try to log in with a regular user.

### 2 Email Address changes ###

When archiving multiple Exchange or Microsoft 365 mailboxes the local part of the primary email address is used as first folder name in the user archive.
When archiving from other supported email servers, the full email address is used as first folder name.
When the primary email address of a user changes, a new folder besides the existing one would be created. Emails already archived into the first folder would be archived in the second one as well and the user would see duplicates when searching over the own archive.

To avoid these duplicates, the following steps should be performed.

#### Prepare username changes ####

Adjust the script MSS_3_prepare_folders.ps1 matching your installation.
Execute the adjusted script.
The folders file is opened automatically in the default editor and can be adjusted according to your planned changes.

#### Apply username changes ####

Adjust the script MSS_4_update_folders.ps1 matching your installation.
Execute the adjusted script.

This script will do:

* Read the user mapping from the text file
* Rename all folders according to the mapping
* The script does merge folders without consent if the target folder already exists. This step cannot be undone easily.

Verify the moved folders manually.

#### Synchronize with the directory service ####

Synchronize users to synchronize the updated email addresses into MailStore. The Sync overview should show changed users.

#### Restart Server ####

Since MailStore Server should be running in Safe Mode, users are not able to log in. Restart the MailStore Server service in the normal mode and try to log in with a regular user.

### 3 Update with old users' names removed in MailStore ###

When the "Delete Users" option was set within MailStore and changed users have been synchronized, in fact new users will be created.
The main disadvantage here is that new and empty archives for these new users have been created and if archiving already has started, duplicates have been created in these new archive. Additionally, privileges on other archives are lost since the old user and settings were removed and new users have been created.

The only smart thing scripts can do here is to merge the old archive into the new one. As written above, this may lead to duplicates in the same archive if archiving into the new archive has already been started.

#### Prepare archive name changes ####

Adjust the script MSS_5_prepare_archives_after_delete.ps1 matching your installation.
Execute the adjusted script.
Open the newly created text file in the user folder and adjust according to your planned changes.

#### Apply archive name changes ####

Adjust the script MSS_6_update_archives_after_delete.ps1 matching your installation.
Execute the adjusted script.

This script will do:

* Read the archive mapping from the text file
* Rename or merge all archives according to the mapping
* The script does merge folders without consent if the target folder already exists. This step cannot be undone easily.

Verify the moved or merged folders manually.

#### Set folder permissions ####

Since the old users have been removed, all folder permissions have to be set again.
This is a manual step and cannot be done automatically.

#### Restart Server ####

Since MailStore Server should be running in Safe Mode, users are not able to log in. Restart the MailStore Server service in the normal mode and try to log in with a regular user.
